# frozen_string_literal: true
module Steam
  # Represents a Client on the Steam Network. The Client creates a connection
  # to the Steam networks, and send and receives Message objects to that
  # connection.
  #
  # The Client holds a collection of handlers that react to received packets,
  # if they care about the packet data.
  class Client
    # The internal connection to Steam
    attr_reader :connection

    # The Handler for Steam user related events (changing name, login, etc)
    attr_reader :steam_user

    # The Handler for Steam app related events (play tokens, etc)
    attr_reader :steam_apps

    # The Handler for the game coordinator events (playing game, etc)
    attr_reader :game_coordinator

    # A list of tokens requires for the game coordinator to launch games
    attr_reader :connect_tokens

    # The SteamId for the current user, nil if not logged in
    attr_reader :steam_id

    # The session id for the current user, nilt if not logged in
    attr_reader :session_id

    # The plugin engine
    attr_reader :plugins

    def initialize(connection = nil)
      connection ||= Networking::Connection.new(ServerList.new.to_a.sample)
      @connect_tokens = []
      @steam_user = nil
      @steam_apps = nil
      @game_coordinator = nil
      @steam_id = default_steam_id
      @connection = connection
      @plugins = Plugins.new(self)
      init_handlers
    end

    # Called by SteamUser handler when the logon message is to be handled.
    # This must be implemented by the subclass
    def on_logon(_msg)
      raise NotImplementedError
    end

    # Called by SteamUser handler when the user is fully logged in. This must
    # be implemented by the subclass.
    def ready
      raise NotImplementedError
    end

    # Registers a plugin with the Client via the plugin name
    #
    # @param plugin [Symbol,String] the plugin name
    def plugin(plugin)
      @plugins.plugin(plugin.to_sym)
    end

    # Stop and start the Client
    def restart
      stop
      start
    end

    # Disconnect the Client
    def stop
      steam_user.logoff if @steam_id && @session_id
      connection.disconnect
      @heartbeat_thread.join if @heartbeat_thread
      @steam_id = default_steam_id
      @session_id = nil
    end

    # Create a new Connection object, and start listening and handling
    # received packets.
    #
    # @return nil
    def start
      @plugins.load

      connection.open
      connection.each_packet do |packet|
        handle_net_msg(packet)
      end
    end

    # Sends a Message to the underlying Steam connection. If we have an active
    # session, the session id and steam id is automatically associated with
    # the request
    #
    # @param msg [Protocol::Message] the Message to send to Steam
    def send_msg(msg)
      Steam.logger.debug("Sending #{msg.body.class.name} to Steam")

      if msg.proto?
        msg.session_id = session_id
        msg.steam_id = steam_id.to_i
      end

      connection.send_msg(msg).nonzero?
    end

    # Creates a new session for the Client
    #
    # @param steam_id [String]
    # @param session_id [String]
    def create_session(steam_id, session_id)
      @steam_id = CommunityId.new(steam_id)
      @session_id = session_id
    end

    # Update the Client's internal connection token list
    #
    # @param tokens [Array<String>] the new list of tokens
    # @param max [Integer] the amount of tokens to keep
    def update_connect_tokens(tokens, max)
      tokens.each { |t| @connect_tokens.insert(0, t) }
      @connect_tokens = @connect_tokens.take(max)
    end

    # Sets the Client's connection session key. Used for encrypting
    # and decrypting packets
    #
    # @param key [String] the key
    def session_key=(key)
      connection.session_key = key
    end

    # Starts the background heartbeat thread.
    #
    # @param tick [Integer] the time to wait between
    #   heartbeat messages
    def start_heartbeat(tick)
      @heartbeat_thread = Thread.new do |t|
        until connection.disconnected?
          send_msg(heartbeat_message)
          heartbeat
          sleep(tick)
        end
      end
      @heartbeat_thread.abort_on_exception = true
      @heartbeat_thread
    end

    # Called every time client sends an alive message to Steam
    def heartbeat
    end

    # Passes each GC message to any plugins.
    #
    # @param msg [GcProtobufMessage] The message to handle
    def gc_message(msg)
      @plugins.loaded_plugins.each do |plugin|
        send(plugin).gc_message(msg)
      end
    end

    private

    # Instantiates the various handlers, and adds them to the handler
    # collection
    def init_handlers
      @auth = Handler::Auth.new(self)
      @steam_user = Handler::SteamUser.new(self)
      @steam_apps = Handler::SteamApps.new(self)
      @game_coordinator = Handler::GameCoordinator.new(self)

      @handlers = Handler::Collection.new
      @handlers.add(@auth, @steam_user, @steam_apps, @game_coordinator)
    end

    # Handles an incoming packet, if the pack is a multi packet (ie: it contains
    # more than one packet) each packet is extracted and passed back to this
    # method.
    #
    # @param packet [Packet] the packet to handle
    def handle_net_msg(packet)
      return handle_multi(packet) if packet.proto? && packet.multi?
      @handlers.handle(packet)
    end

    # Handles a packet containing one or more other Packets. Sometimes
    # the Packet data will be Gziped, this method handles that case as well.
    #
    # @note this method does not ensure the packet is a multi packet, the
    #   calling method should handle this
    # @param packet [Networking::Packet] the packet to handle
    def handle_multi(packet)
      msg = packet.as_message(Steamclient::CMsgMulti.new)

      body = StringIO.new(msg.body.message_body)
      payload = if msg.body&.size_unzipped&.nonzero?
                  Zlib::GzipReader.new(body)
                else
                  body
                end

      handle_packets_from_multi_payload(payload)
    end

    # Extracts one or more packet from a given payload IO object.
    #
    # @param payload [#read] The payload io object
    def handle_packets_from_multi_payload(payload)
      reader = ByteReader.new(payload)

      until reader.eof?
        packet_length = reader.unsigned_int32
        handle_net_msg(Networking::Packet.new(reader.read(packet_length)))
      end
    end

    # The heartbeat message that we send once the user is connected
    #
    # @return [ProtobufMessage]
    def heartbeat_message
      ProtobufMessage.new(MsgHdrProtoBuf.new,
                          Steamclient::CMsgClientHeartBeat.new,
                          EMsg::CLIENT_HEART_BEAT)
    end

    # The default SteamID to be sent with each message before a user
    # has been authed
    # @return [Steam2Id]
    def default_steam_id
      Steam2Id.new(EUniverse::PUBLIC, 0, 0)
    end
  end
end
