# frozen_string_literal: true
require 'openssl'
require 'zlib'
require 'base64'

module Steam
  # Handles Steam AES key encryption and decrptyion of raw packet data.
  class Crypto
    # The Steam public key
    KEY = 'MIGdMA0GCSqGSIb3DQEBAQUAA4GLADCBhwKBgQDf7BrWLBBmLBc1OhSwfFkRf53'\
          'T2Ct64+AVzRkeRuh7h3SiGEYxqQMUeYKO6UWiSRKpI2hzic9pobFhRr3Bvr/WARvY'\
          'gdTckPv+T1JzZsuVcNfFjrocejN1oWI0Rrtgt4Bo+hOneoo3S57G9F1fOpn5nsQ66'\
          'WOiu4gZKODnFMBCiQIBEQ=='

    # Encrypt a given string with a given key, get the encrypted data back.
    #
    # @param data [String] the data to encrypt
    # @param key [String] the key to encrypt the data with
    def self.encrypt(data, key)
      new.encrypt(StringIO.new(data), key).string
    end

    # Decrypt a given string with a given key, get the decrypted data back.
    #
    # @param data [String] the data to decrypt
    # @param key [String] the key to decrypt the data with
    def self.decrypt(data, key)
      new.decrypt(StringIO.new(data), key).string
    end

    # Generates a tuple representing the session key, both plain text and
    # encrypted
    #
    # @example Generating a key
    #   crypto = Cyrpto.new
    #   crypted_key, key = crypto.generate_key
    #
    # @return [Array] the crypted_key, plain_key
    def generate_key
      key   = OpenSSL::PKey::RSA.new(Base64.strict_decode64(KEY))
      plain = Random.new.bytes(32)
      padding = OpenSSL::PKey::RSA::PKCS1_OAEP_PADDING

      [key.public_encrypt(plain, padding), plain]
    end

    # Encrypts the given IO stream using the given key
    #
    # @example Encrypting data
    #   crypto = Crypto.new
    #   crypto.encrypt('data', 'key')
    #
    # @param io [#read] The IO object to encrypt
    # @param key [String] The key
    # @return [:read] an IO object represeting the encrypted data
    def encrypt(io, key)
      iv = Random.new.bytes(16)

      iv_cipher = iv_cipher(key, :encrypt)
      cipher    = data_cipher(key, iv, :encrypt)

      StringIO.new(iv_cipher.update(iv) + iv_cipher.final +
                   cipher.update(io.read) + cipher.final)
    end

    # Decrypts the given IO stream using the given key
    #
    # @example Encrypting data
    #   crypto = Crypto.new
    #   crypto.decrypt(StringIO.new('data'), 'key')
    #
    # @param io [#read] The IO object to encrypt
    # @param key [String] The key
    # @return [:read] an IO object represeting the decrypted data
    def decrypt(io, key)
      iv_cipher = iv_cipher(key, :decrypt)
      crypted_iv = io.read(16)
      iv = iv_cipher.update(crypted_iv) + iv_cipher.final

      cipher = data_cipher(key, iv, :decrypt)

      StringIO.new(cipher.update(io.read) + cipher.final)
    end

    # Get the 32 bit crc for the given key
    #
    # @param key [String] The key
    # @return [Integer] the crc
    def session_key_crc(key)
      Zlib.crc32(key)
    end

    private

    # @api private
    def data_cipher(key, iv, type)
      cipher = OpenSSL::Cipher::AES.new(256, :CBC)
      cipher.send(type)
      cipher.key = key
      cipher.iv = iv
      cipher
    end

    # @api private
    def iv_cipher(key, type)
      iv_cipher = OpenSSL::Cipher::AES.new(256, :ECB)
      iv_cipher.send(type)
      iv_cipher.key = key
      iv_cipher.padding = 0
      iv_cipher
    end
  end
end
