# frozen_string_literal: true
module Steam
  module Handler
    # Steam Auth handler. Handles channel encryption and decryption.
    #
    # @example Using the handler inside Handler::Collection
    #   collection = Handler::Collection.new
    #   collection << Handler::Auth.new
    #   collection.handle(packet)
    #
    # @example Using the handler stand alone
    #   handler = Handler::Auth.new
    #   handler.handle_packet(packet)
    class Auth
      include Handler::Base

      # Respond the Channel encrypt requests and results
      handles EMsg::CHANNEL_ENCRYPT_REQUEST, EMsg::CHANNEL_ENCRYPT_RESULT

      # Override the constructor to set our key to nil. This key will be
      # used if the encryption is a success to decode and encode all future
      # packet data.
      def initialize(*)
        super
        @key = nil
      end

      # Handles the given packet. This handler is only concerned about the
      # channel encryption request and result.
      #
      # @param packet [Networking::Packet] the packet to handle
      def handle(packet)
        case packet.msg_type
        when EMsg::CHANNEL_ENCRYPT_REQUEST
          handle_channel_encrypt_request(packet)
        when EMsg::CHANNEL_ENCRYPT_RESULT
          handle_channel_encrypt_result(packet)
        end
      end

      private

      # Decodes the packet as a MsgChannelEncryptResult. If the result is
      # a success the client's connection session key is set.
      #
      # @param packet [Networking::Packet]
      def handle_channel_encrypt_result(packet)
        msg = packet.as_message(MsgChannelEncryptResult.new)
        raise 'encrypt result failed' unless msg.body.result == EResult::OK

        # Set the session key
        client.session_key = @key
        client.ready
        Steam.logger.debug('Client channel encrypted')
        true
      end

      # Decodes the packet as a MsgChannelEncryptResponse. A key is generated
      # and sent to the steam servers. If the encryption request is a success
      # the connection's session key is set in the handling of the result
      #
      # @param _packet [Networking::Packet]
      def handle_channel_encrypt_request(_packet)
        encrypt = Crypto.new
        crypted_key, key = encrypt.generate_key

        msg = ClientMessage.new(MsgHdr.new,
                                MsgChannelEncryptResponse.new,
                                EMsg::CHANNEL_ENCRYPT_RESPONSE)
        msg.payload.write(crypted_key)
        msg.payload.write_int64(encrypt.session_key_crc(crypted_key))

        send_msg(msg)
        @key = key
      end
    end
  end
end
