# frozen_string_literal: true

require 'securerandom'
module Steam
  module Handler
    # Game coordinator handler
    class GameCoordinator
      include Handler::Base

      # Describes the EMsgs this Handler cares about
      handles EMsg::CLIENT_FROM_GC

      # Send a message to the Game Coordinator
      #
      # @note you must have asked and been given a GC session
      # @todo remove hardcoded app id
      #
      # @param msg [GcMessage] The message to send
      # @return [Bool]
      def send_msg(msg)
        gcmsg = Steamclient::CMsgGCClient.new
        gcmsg.appid = 730
        gcmsg.msgtype = msg.header.msg
        gcmsg.payload = msg.encode

        header = MsgHdrProtoBuf.new
        # header.proto.routing_appid = 730
        gc = ProtobufMessage.new(header, gcmsg, EMsg::CLIENT_TO_GC)

        client.send_msg(gc)
      end

      # Handle a packet
      #
      # @param packet [Networking::Packet] the packet to handle
      def handle(packet)
        case packet.msg_type
        when EMsg::CLIENT_FROM_GC
          handle_gc_message(packet)
        end
      end

      # Tell the Steam servers that the Client is playing a game.
      #
      # @param game_id [Integer] the game to play
      #
      # @example Playing CS:GO
      #   game_coordinator = client.game_coordinator
      #   game_coordinator.play(730, "CS:GO")
      def play(game_id)
        game = Steamclient::CMsgClientGamesPlayed.new

        played = Steamclient::CMsgClientGamesPlayed::GamePlayed.new
        played.game_id = game_id
        played.token = @client.connect_tokens.shift

        game.games_played ||= [played]

        msg = ProtobufMessage.new(MsgHdrProtoBuf.new, game,
                                  EMsg::CLIENT_GAMES_PLAYED_WITH_DATA_BLOB)
        client.send_msg(msg)
      end

      private

      # @api private
      def handle_gc_message(packet)
        msg = packet.as_message(Steamclient::CMsgGCClient.new)

        gcmsg = GcProtobufMessage.new(
          MsgGCHdrProtoBuf.new,
          msg.body,
          msg.body.msgtype
        )

        client.gc_message(gcmsg)
      end
    end
  end
end
