# frozen_string_literal: true
require 'logger'

module Steam
  # Small wrapper around the Ruby Logger class. A `STEAM_LOG_LEVEL`
  # environment variable may also dictate the log level.
  #
  # @example Creating a Logger object
  #   logger = Logger.new(STDOUT)
  #   logger.level = Logger::WARN
  class Logger < ::Logger
    # Raised when the #env_key env variable is not supported
    InvalidLogLevel = Class.new(StandardError)

    # Log level WARN
    WARN  = ::Logger::WARN
    # Log level INFO
    INFO  = ::Logger::INFO
    # Log level ERROR
    ERROR = ::Logger::ERROR
    # Log level DEBUG
    DEBUG = ::Logger::DEBUG
    # Log level FATAL
    FATAL = ::Logger::FATAL

    # Create a Logger that logs to a given IO object
    #
    # @param io [#read] The io object
    def initialize(io)
      super(io)
      self.level = _level
    end

    private

    # @api private
    def _level
      level = ENV[env_key]
      level = 'DEBUG' if level.nil? || level.empty?
      unless allowed_levels.include?(level.to_s)
        raise InvalidLogLevel, "log level #{level} not allowed, must be one of"\
                  "#{allowed_levels}"
      end
      self.class.const_get("Logger::#{level}")
    end

    # @api private
    def allowed_levels
      %w(WARN INFO ERROR DEBUG FATAL)
    end

    def env_key
      'STEAM_LOG_LEVEL'
    end
  end
end
