# frozen_string_literal: true
module Steam
  # Allows "plugins" via gems. The plugins are just gems that react to client
  # events an maybe expose an additional api. For example a CSGO plugin would
  # be respoinsible for strictly CSGO related things and could have syntatic
  # candy for things such as logging the player on, and handling the connection
  # flow of CSGO related connection events. It could also provide the abililty
  # to send specific GC messages like the requesting of match info.
  #
  # The plugins are refered to by name, ie: csgo
  #
  # The CSGO plugin would be a gem that is named `steam-csgo` and has the
  # following struture.
  #
  #  -steam-csgo
  #    - lib
  #      - steam
  #        - csgo
  #          - base.rb
  #        - csgo.rb
  #    - ... other gem things ...
  #
  # The fastpeek/steam-csgo gem is a CSGO plugin for references
  class Plugins
    include Enumerable

    attr_reader :client, :loaded_plugins

    # Instantiate a new Plugins engine
    def initialize(client)
      @client = client
      @mutex = Mutex.new
      @plugins = []
      @loaded_plugins = []
      @loaded = false
    end

    # Determine if the Client has a plugin loaded
    #
    # @param plugin [Symbol]
    # @return [Bool]
    def loaded?(plugin)
      @loaded_plugins.include?(plugin.to_sym)
    end

    # Add a plugin to the internal list. Plugins are not loaded until
    # #load is called.
    def plugin(plugin)
      @plugins << plugin.to_sym
      @plugins
    end

    # Load the plugins. Attempt to require each plugin. If the require call
    # fails an error message is printed.
    #
    # If the client has already loaded the plugins, they are not loaded again.
    #
    # @return [Integer] the number of plugins it loaded
    def load
      return true if @loaded
      @loaded_plugins = require_plugins
      Steam.logger.debug("Loaded plugins: #{loaded_plugins}")
      setup_plugin_helpers(loaded_plugins)
      @loaded = true
      @loaded_plugins.count >= 1
    end

    private

    # @api private
    def setup_plugin_helpers(loaded_plugins)
      loaded_plugins.each { |plugin| define_plugin_ivar(plugin) }
    end

    # @api private
    def define_plugin_ivar(plugin)
      client.class.class_eval do
        define_method plugin do
          var = instance_variable_get("@#{plugin}")

          unless var
            val = "Steam::#{plugin.capitalize}::Base".constantize.new(self)
            var = instance_variable_set("@#{plugin}", val)
          end
          var
        end
      end
    end

    # @api private
    def require_plugins
      @plugins.map { |plugin| _require(plugin) }.compact
    end

    # @api private
    def _require(plugin)
      require "steam/#{plugin}"
      plugin
    rescue LoadError
      msg = "Failed to load steam-#{plugin} plugin. Is it in your Gemfile?"
      Steamd.logger.error(msg)
      nil
    end
  end
end
