# frozen_string_literal: true
module Steam
  module Protocol
    # The base Steam message. Each message contains a header, body and emsg
    # attribute.
    #
    # The #emsg corresponds to the constants EMsg in the SteamLanguage
    #
    # @see ClientMessage
    # @see ProtobufMessage
    module Message
      # Valve's mask for determining if a packet is Protobuf backed
      PROTO_MASK = 0x80000000

      # Each packet has a header. The header is defined by the Steam Language.
      # A header can be a MsgHdr, or a MsgHdrProtoBuf
      #
      # @see MsgHdrProtoBuf
      # @see MsgHdr
      attr_reader :header

      # Each packet has a body, this is the Message being held in the packet.
      # @see Message
      attr_reader :body

      # Each packet has an optional byte string payload
      attr_reader :payload

      # The Message type
      #
      # @see EMsg
      attr_reader :emsg

      # Instantiate a Message object
      #
      # @param header [MsgHdr, MsgHdrProtoBuf]
      # @param body
      # @param emsg
      def initialize(header, body, emsg)
        @header = header
        @body = body
        @header.msg = emsg
        @payload = ByteWriter.new
        @emsg = emsg
        nil
      end

      # By default a Message is not Protobuf backed
      def proto?
        false
      end

      # The steam id associated with the message
      #
      # @param _sid [String] the Steam id
      def steam_id=(_sid)
        raise NotImplementedError
      end

      # The session id associated with the message
      #
      # @param _sid [String] the Session id
      def session_id=(_sid)
        raise NotImplementedError
      end

      # Decode a Packet from an io object
      #
      # @param io [#read] The IO stream
      def decode(io)
        @header.decode_from(io)

        if @body.respond_to?(:decode_from)
          # Steam language
          @body.decode_from(io)
        else
          # Proto
          @body = @body.class.decode(io.read)
        end
      end

      # Encode a Packet from to a string
      #
      # @see Packet
      # @return [String] The byte representation of the Packet
      def encode
        io = StringIO.new
        io.set_encoding('BINARY')

        @header.encode_to(io)
        body = @body.encode
        io.write(body)
        io.write(@payload.string)

        io.string
      end
    end
  end
end
